package com.stl.flutter_ali_oauth;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;

import com.stl.flutter_ali_oauth.imp.FxchatOneKeyHelper;
import com.stl.flutter_ali_oauth.plugin.IOneKeyHelper;

import java.lang.ref.WeakReference;

import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * STLAliOauthPlugin
 */
public class FlutterAliOauthPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;
    protected final static String CHANNEL_NAME = "com.stl/stl_ali_oauth";
    private static WeakReference<Activity> activity;
    private static Context context;
    private IOneKeyHelper oneKeyHelper;
    private static WeakReference<BinaryMessenger> binaryMessenger;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        Log.d("FlutterAliOauthPlugin","onAttachedToEngine");
        context = flutterPluginBinding.getApplicationContext();
        binaryMessenger = new WeakReference<BinaryMessenger>(flutterPluginBinding.getFlutterEngine().getDartExecutor());
        channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), CHANNEL_NAME);
        channel.setMethodCallHandler(this);
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    public static void registerWith(Registrar registrar) {
        Log.d("FlutterAliOauthPlugin","registerWith");
        final MethodChannel channel = new MethodChannel(registrar.messenger(), CHANNEL_NAME);
        binaryMessenger = new WeakReference<>(registrar.messenger());
        context=registrar.context().getApplicationContext();
        channel.setMethodCallHandler(new FlutterAliOauthPlugin());
    }

    public static Activity getActivity() {
        return activity.get();
    }

    public static Context getApplicationContext() {
        return context;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case "init":
                init(call, result);
                break;
            case "preLogin":
                preLogin(call, result);
                break;
            case "login":
                login(call, result);
                break;
            case "checkVerifyEnable":
                checkVerifyEnable(call, result);
                break;
            case "quitLoginPage":
                quitLoginPage();
                break;
            case "hideLoginLoading":
                hideLoginLoading();
                break;
            case "isShow":
                isShow(call, result);
            case "getVerifyToken":
                getVerifyToken(call, result);
                break;
            default:
                throw new IllegalArgumentException("Unkown operation" + call.method);
        }
    }

    private void init(final MethodCall call, final MethodChannel.Result methodResult) {
        Log.d("FlutterAliOauthPlugin","init");
        //生成一键登录,初始化消息渠道
        oneKeyHelper = FxchatOneKeyHelper.getInstance();
        oneKeyHelper.initEventChannel(binaryMessenger.get());
        //初始化
        oneKeyHelper.init(call, methodResult);
    }

    private void preLogin(final MethodCall call, final MethodChannel.Result methodResult) {
        Log.d("FlutterAliOauthPlugin","init");
        oneKeyHelper.preLogin(call, methodResult);
    }

    private void login(final MethodCall call, final MethodChannel.Result methodResult) {
        Log.d("FlutterAliOauthPlugin","init");
        oneKeyHelper.login(call, methodResult);
    }

    private void checkVerifyEnable(final MethodCall call, final MethodChannel.Result methodResult) {
        Log.d("FlutterAliOauthPlugin","init");
        oneKeyHelper.checkVerifyEnable(call, methodResult);
    }

    private void quitLoginPage() {
        Log.d("FlutterAliOauthPlugin","init");
        oneKeyHelper.quitLoginPage();
    }

    private void hideLoginLoading() {
        Log.d("FlutterAliOauthPlugin","init");
        oneKeyHelper.hideLoginLoading();
    }

    private void getVerifyToken(final MethodCall call, final MethodChannel.Result methodResult) {
        Log.d("FlutterAliOauthPlugin","init");
        oneKeyHelper.getVerifyToken(call, methodResult);
    }

    private void isShow(final MethodCall call, final MethodChannel.Result methodResult) {
        oneKeyHelper.isShow(call, methodResult);
    }


    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        Log.d("FlutterAliOauthPlugin","onDetachedFromEngine");
        channel.setMethodCallHandler(null);
    }


    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        Log.d("FlutterAliOauthPlugin","onAttachedToActivity");
        activity = new WeakReference<Activity>(binding.getActivity());
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        Log.d("FlutterAliOauthPlugin","onDetachedFromActivityForConfigChanges");
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        Log.d("FlutterAliOauthPlugin","onReattachedToActivityForConfigChanges");
    }

    @Override
    public void onDetachedFromActivity() {
        Log.d("FlutterAliOauthPlugin","onDetachedFromActivity");
    }
}
