package com.stl.flutter_ali_oauth.helper;

import android.content.Context;
import android.media.session.MediaSession;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.IntDef;

import com.google.gson.Gson;
import com.mobile.auth.gatewayauth.PhoneNumberAuthHelper;
import com.mobile.auth.gatewayauth.PreLoginResultListener;
import com.mobile.auth.gatewayauth.TokenResultListener;
import com.mobile.auth.gatewayauth.model.TokenRet;
import com.stl.flutter_ali_oauth.FlutterAliOauthPlugin;
import com.stl.flutter_ali_oauth.bean.TokenResult;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

public class STLPhoneNumberAuthHelper {
    private static final String TAG = "PhoneNumberAuthHelper";

    /**
     * 打开授权页面成功
     */
    public static final String CODE_CALL_AUTHORIZE = "600001";
    /**
     * 唤起授权页失败
     */
    public static final String CODE_CALL_AUTHORIZE_FAIL = "600002";
    /**
     * 未检测到sim卡
     */
    public static final String CODE_NOT_SIM = "600007";
    /**
     * 蜂窝网络未开启
     */
    public static final String CODE_NOT_PHONE_NET = "600008";
    /**
     * 未知异常
     */
    public static final String CODE_UNKONW_EXP = "600010";
    /**
     * 获取token失败
     */
    public static final String CODE_GET_TOKEN_FAIL = "600011";
    /**
     * 预取号失败
     */
    public static final String CODE_PRE_GET_PHONE_FAIL = "600012";
    /**
     * 接口超时
     */
    public static final String CODE_API_TIMEOUT = "600015";
    /**
     * 用户已切换上网卡
     */
    public static final String CODE_CHANG_SIM = "600021";

    /**
     * 用户取消登录
     */
    public static final String CODE_CANCEL_LOGIN = "700000";

    /**
     * 网络无连接
     */
    public static final String CODE_NOT_NET = "-8001";

    /**
     * 当前网络不支持，请检测蜂窝网络后重试
     */
    public static final String CODE_NOT_SUPPOR = "10000";


    private static STLPhoneNumberAuthHelper instance;


    private PhoneNumberAuthHelper mAlicomAuthHelper;
    private TokenResultListener mTokenListener;
    private String token;
    private boolean checkRet;
    private Context context;
    private boolean isInit;
    private boolean isShow;
    private int type=Type.LOGIN;

    public static STLPhoneNumberAuthHelper getInstance() {
        if (instance == null) {
            instance = FxPhoneNumberAuthHelperInner.phoneNumberAuthHelper;
        }
        return instance;
    }

    private STLPhoneNumberAuthHelper() {

    }

    private static class FxPhoneNumberAuthHelperInner {
        private static STLPhoneNumberAuthHelper phoneNumberAuthHelper = new STLPhoneNumberAuthHelper();
    }

    public void init(Context context, final String sk, final STLTokenResultListener stlTokenResultListener) {
        if (isInit) {
            return;
        }
        isShow = false;
        this.context = context;
        /*
         *   1.init get token callback Listener
         */
        mTokenListener = new TokenResultListener() {
            @Override
            public void onTokenSuccess(final String ret) {
                Log.i(TAG, "onTokenSuccess: " + ret);
                new Handler(Looper.getMainLooper()).post(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            TokenRet tokenRet = null;
                            tokenRet = new Gson().fromJson(ret, TokenRet.class);

                            if (tokenRet != null && CODE_CALL_AUTHORIZE.equals(tokenRet.getCode())) {
                                //页面唤起成功
                                isShow = true;
                            }
                            TokenResult tokenResult = null;
                            if (tokenRet != null) {
                                tokenResult = new TokenResult();
                                tokenResult.setSuccess(true);
                                tokenResult.setType(type);
                                tokenResult.setCode(tokenRet.getCode());
                                tokenResult.setMsg(tokenRet.getMsg());
                                tokenResult.setToken(tokenRet.getToken());
                                tokenResult.setVendorName(tokenRet.getVendorName());
                            }
                            if (stlTokenResultListener != null) {
                                String tokenResultString = new Gson().toJson(tokenResult);
                                stlTokenResultListener.onTokenSuccess(tokenResultString);
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                });
            }

            @Override
            public void onTokenFailed(final String ret) {
                Log.i(TAG, "onTokenFailed: " + ret);
                new Handler(Looper.getMainLooper()).post(new Runnable() {
                    @Override
                    public void run() {
                        mAlicomAuthHelper.hideLoginLoading();
                        try {

                            TokenRet tokenRet = null;
                            tokenRet = new Gson().fromJson(ret, TokenRet.class);

                            TokenResult tokenResult = null;
                            if (tokenRet != null) {
                                tokenResult = new TokenResult();
                                tokenResult.setSuccess(false);
                                tokenResult.setType(type);
                                tokenResult.setCode(tokenRet.getCode());
                                tokenResult.setMsg(tokenRet.getMsg());
                                tokenResult.setToken(tokenRet.getToken());
                                tokenResult.setVendorName(tokenRet.getVendorName());
                            }
                            if (stlTokenResultListener != null) {
                                String tokenResultString = new Gson().toJson(tokenResult);
                                stlTokenResultListener.onTokenFailed(tokenResultString);
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                });
            }
        };

        /*
         *   2.init AlicomAuthHelper with tokenListener
         */
        mAlicomAuthHelper = PhoneNumberAuthHelper.getInstance(context, mTokenListener);
        //设置SDK秘钥
        mAlicomAuthHelper.setAuthSDKInfo(sk);

        mAlicomAuthHelper.setAuthListener(mTokenListener);
        if (!checkVerifyEnable() && stlTokenResultListener != null) {
            Log.i(TAG, "checkVerifyEnable():"+checkVerifyEnable());
            TokenResult result = new TokenResult();
            result.setCode(CODE_NOT_SUPPOR);
            result.setSuccess(false);
            result.setType(type);
            result.setVendorName("");
            result.setMsg("当前网络不支持，请检测蜂窝网络后重试");
            result.setToken("");
            String resultString = new Gson().toJson(result);
            stlTokenResultListener.onTokenFailed(resultString);
        }
        mAlicomAuthHelper.setLoggerEnable(true);
        isInit = true;
    }

    /**
     * 预取号码信息，建议在 APP 登录页初始化时调用，提高后续授权页的打开速度。
     * 不要在 App 启动初始化时调用，避免接口调用资源浪费
     */
    public void accelerateLoginPage(int timeout, final STLPreLoginResultListener stlPreLoginResultListener) {
        if (mAlicomAuthHelper!=null){
            mAlicomAuthHelper.accelerateLoginPage(timeout, new PreLoginResultListener() {
                @Override
                public void onTokenSuccess(final String vendor) {
                    Log.i(TAG, "onTokenSuccess:预取号成功 ");

                    if (stlPreLoginResultListener != null) {
                        stlPreLoginResultListener.onTokenSuccess(vendor);
                    }
                }

                @Override
                public void onTokenFailed(final String vendor, final String ret) {
                    Log.i(TAG, vendor + "预取号失败:\n" + ret);

                    if (stlPreLoginResultListener != null) {
                        stlPreLoginResultListener.onTokenFailed(vendor, ret);
                    }
                }
            });
        }
    }

    /**
     * 唤起一键登录授权页面，在用户授权后获取一键登录的token
     *
     * @param loginLayout
     * @param timeout
     */
    public void getLoginToken(Context context, int type,STLAuthLoginLayout loginLayout, int timeout) {
        if (mAlicomAuthHelper != null) {
            this.context = context;
            isShow = false;
            this.type=type;
             if (type==Type.LOGIN){
                loginLayout.onLoginLayout(mAlicomAuthHelper);
            }else if (type==Type.VERIFY){
                loginLayout.onVerifyLayout(mAlicomAuthHelper);
            }
            mAlicomAuthHelper.getLoginToken(context, timeout);
        }
    }

    /**
     * 获取一键验证token
     * 不包括界面
     * @param timeout
     */
    public void getVerifyToken(int timeout){
        if (mAlicomAuthHelper != null){
            mAlicomAuthHelper.getVerifyToken(timeout);
        }
    }

    public void quitLoginPage() {
        if (mAlicomAuthHelper != null) {
            mAlicomAuthHelper.quitLoginPage();
        }
    }

    public void hideLoginLoading() {
        if (mAlicomAuthHelper != null) {
            mAlicomAuthHelper.hideLoginLoading();
        }
    }

    public boolean isShow() {
        return isShow;
    }

    /**
     * 检查运营商是否合规，不支持境外卡一键登录
     */
    public boolean checkCarrierName() {

        boolean isTrue = false;

        String currentCarrierName;

        if (mAlicomAuthHelper != null) {
            currentCarrierName = mAlicomAuthHelper.getCurrentCarrierName();
            Log.d("currentCarrierName", "" + currentCarrierName);
            isTrue = currentCarrierName == "CMCC" || currentCarrierName == "CUCC" || currentCarrierName == "CTCC";
        }

        return isTrue;
    }

    /**
     * SDK 判断网络环境是否支持
     */
    public boolean checkVerifyEnable() {

        boolean checkEnvAvailable = false;

        if (mAlicomAuthHelper != null) {
            Log.d("checkVerifyEnable", "mAlicomAuthHelper.checkEnvAvailable():" +mAlicomAuthHelper.checkEnvAvailable());
            Log.d("checkVerifyEnable", "checkCarrierName():" +checkCarrierName());
            checkEnvAvailable = mAlicomAuthHelper.checkEnvAvailable() && checkCarrierName();
        }

        return checkEnvAvailable;
    }

    public void onDestroy() {
    }


    @IntDef({Type.LOGIN, Type.VERIFY})
    @Retention(RetentionPolicy.SOURCE)
    public @interface Type {
        int LOGIN = 1;
        int VERIFY = 2;
    }

}
