package com.stl.flutter_ali_oauth.imp;

import android.content.Context;
import android.content.pm.ActivityInfo;
import android.os.Build;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.mobile.auth.gatewayauth.AuthRegisterXmlConfig;
import com.mobile.auth.gatewayauth.AuthUIConfig;
import com.mobile.auth.gatewayauth.AuthUIControlClickListener;
import com.mobile.auth.gatewayauth.PhoneNumberAuthHelper;
import com.mobile.auth.gatewayauth.ui.AbstractPnsViewDelegate;
import com.stl.flutter_ali_oauth.FlutterAliOauthPlugin;
import com.stl.flutter_ali_oauth.R;
import com.stl.flutter_ali_oauth.bean.AccelerateTokenResult;
import com.stl.flutter_ali_oauth.bean.TokenResult;
import com.stl.flutter_ali_oauth.bean.ViewMessageResult;
import com.stl.flutter_ali_oauth.config.Config;
import com.stl.flutter_ali_oauth.event.IMessageEvent;
import com.stl.flutter_ali_oauth.event.TokenEvent;
import com.stl.flutter_ali_oauth.event.ViewEvent;
import com.stl.flutter_ali_oauth.helper.STLPhoneNumberAuthHelper;
import com.stl.flutter_ali_oauth.helper.STLPreLoginResultListener;
import com.stl.flutter_ali_oauth.helper.STLTokenResultListener;
import com.stl.flutter_ali_oauth.plugin.IOneKeyHelper;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

/**
 * @author Ricky
 * @date 2020/10/21
 * Fxchat项目 帮助类
 */
public class FxchatOneKeyHelper implements IOneKeyHelper, View.OnClickListener {


    private static FxchatOneKeyHelper instance;
    private STLPhoneNumberAuthHelper helper;
    private final String TAG = "FxchatOneKeyHelper";
    private MethodChannel.Result loginMethodResult;
    private IMessageEvent<ViewMessageResult> viewEvent;
    private IMessageEvent<String> tokenEvent;

    private int type= STLPhoneNumberAuthHelper.Type.LOGIN;

    private final int weChat = 1;
    private final int qq = 2;
    private final int weibo = 3;
    private final int other_login=4;
    private final int back=5;
    private final int other_phone_verify=6;

    private Map<String,String> loginMessage=new HashMap<>();
    private Map<String,String> verifyMessage=new HashMap<>();

    public static FxchatOneKeyHelper getInstance() {
        if (instance == null) {
            instance = new FxchatOneKeyHelper();
        }
        return instance;
    }

    private FxchatOneKeyHelper() {
    }


    @Override
    public void init(MethodCall call, final MethodChannel.Result methodResult) {
        Log.i(TAG, "init");
        String sk = call.argument(Config.ALI_AUTH_SK);
        Log.i(TAG, "sk:" + sk);
        helper = STLPhoneNumberAuthHelper.getInstance();
        loginMethodResult = methodResult;
        helper.init(FlutterAliOauthPlugin.getApplicationContext(), sk, new STLTokenResultListener() {

            @Override
            public void onTokenSuccess(String tokenResult) {
                Log.i(TAG, "init onTokenSuccess result:" + tokenResult);
                tokenEvent.sendSuccess(tokenResult);
                loginMethodResult.success(tokenResult);
            }

            @Override
            public void onTokenFailed(String tokenResult) {
                Log.i(TAG, "init onTokenFailed result:" + tokenResult);
                tokenEvent.sendSuccess(tokenResult);
//                TokenResult result = new Gson().fromJson(tokenResult, TokenResult.class);
                loginMethodResult.success(tokenResult);
            }
        });
    }

    @Override
    public void preLogin(MethodCall call, final MethodChannel.Result methodResult) {
        int timeOut = 10000;
        if (call.hasArgument(Config.ALI_AUTH_TIME_OUT)) {
            timeOut = call.argument(Config.ALI_AUTH_TIME_OUT);
        }
        helper.accelerateLoginPage(timeOut, new STLPreLoginResultListener() {
            @Override
            public void onTokenSuccess(String vendor) {
                AccelerateTokenResult tokenResult = new AccelerateTokenResult();
                tokenResult.setSuccess(true);
                tokenResult.setVendor(vendor);
                tokenResult.setCode(STLPhoneNumberAuthHelper.CODE_CALL_AUTHORIZE);
                tokenResult.setMsg("预取号成功!");
                tokenResult.setRet("");
                String tokenResultString = new Gson().toJson(tokenResult);
                Log.i(TAG, "preLogin onTokenSuccess result:" + tokenResultString);
                methodResult.success(tokenResultString);
            }

            @Override
            public void onTokenFailed(String vendor, String ret) {
                AccelerateTokenResult tokenResult = new AccelerateTokenResult();
                tokenResult.setSuccess(false);
                tokenResult.setVendor(vendor);
                tokenResult.setCode(STLPhoneNumberAuthHelper.CODE_PRE_GET_PHONE_FAIL);
                tokenResult.setMsg("预取号失败!");
                tokenResult.setRet(ret);
                String tokenResultString = new Gson().toJson(tokenResult);
                Log.i(TAG, "preLogin onTokenFailed result:" + tokenResultString);
                methodResult.success(tokenResultString);
            }
        });
    }

    @Override
    public void login(MethodCall call, MethodChannel.Result methodResult) {
        Log.i(TAG, "login");
        if (helper != null) {
            loginMethodResult = methodResult;
            int timeOut = 10000;
            if (call.hasArgument(Config.ALI_AUTH_TIME_OUT)) {
                timeOut = call.argument(Config.ALI_AUTH_TIME_OUT);
            }

            if (call.hasArgument(Config.ALI_AUTH_TYPE)) {
                type = call.argument(Config.ALI_AUTH_TYPE);
            }else {
                type = STLPhoneNumberAuthHelper.Type.LOGIN;
            }
            if (call.hasArgument(Config.ALI_AUTH_LOGIN_MESSAGE)){
                loginMessage.clear();
                Map<String,String> map=call.argument(Config.ALI_AUTH_LOGIN_MESSAGE);
                Log.i(TAG, "map:"+map.toString());
                loginMessage.putAll(map);
            }else if (call.hasArgument(Config.ALI_AUTH_VERIFY_MESSAGE)){
                verifyMessage.clear();
                Map<String,String> map=call.argument(Config.ALI_AUTH_VERIFY_MESSAGE);
                Log.i(TAG, "map:"+map.toString());
                verifyMessage.putAll(map);
            }

            helper.getLoginToken(FlutterAliOauthPlugin.getApplicationContext(), type, this, timeOut);
        }
    }

    @Override
    public void checkVerifyEnable(MethodCall call, MethodChannel.Result methodResult) {
        boolean enable = helper.checkVerifyEnable();
        methodResult.success(enable);
    }

    @Override
    public void getVerifyToken(MethodCall call, MethodChannel.Result methodResult) {
        if (helper != null) {
            loginMethodResult = methodResult;
            int timeOut = 10000;
            if (call.hasArgument(Config.ALI_AUTH_TIME_OUT)) {
                timeOut = call.argument(Config.ALI_AUTH_TIME_OUT);
            }
            helper.getVerifyToken(timeOut);
        }
    }

    @Override
    public void quitLoginPage() {
        if (helper != null) {
            helper.quitLoginPage();
        }
    }

    @Override
    public void hideLoginLoading() {
        if (helper != null) {
            helper.hideLoginLoading();
        }
    }

    @Override
    public void isShow(MethodCall call, MethodChannel.Result methodResult) {
        if (helper != null) {
            methodResult.success(helper.isShow());
        } else {
            methodResult.success(false);
        }
    }

    @Override
    public void initEventChannel(BinaryMessenger messenger) {
        //初始化view推送
        viewEvent = ViewEvent.getInstance();
        viewEvent.initEventChannel(messenger);
        //初始化token推送
        tokenEvent= TokenEvent.getInstance();
        tokenEvent.initEventChannel(messenger);
    }

    @Override
    public void onVerifyLayout(PhoneNumberAuthHelper phoneNumberAuthHelper) {
        phoneNumberAuthHelper.removeAuthRegisterXmlConfig();
        phoneNumberAuthHelper.removeAuthRegisterViewConfig();
        phoneNumberAuthHelper.addAuthRegisterXmlConfig(new AuthRegisterXmlConfig.Builder()
                .setLayout(R.layout.activity_one_key_verify, new AbstractPnsViewDelegate() {
                    @Override
                    public void onViewCreated(View view) {
                        //其他手机号验证
                        TextView tvOtherLogin = (TextView) findViewById(R.id.tv_one_key_verify_other_phone);
                        tvOtherLogin.setOnClickListener(FxchatOneKeyHelper.this);

                        //手机验证
                        TextView tvPhoneVerify = (TextView) findViewById(R.id.tv_one_key_verify_phone_verify);
                        String phoneVerify=verifyMessage.get("phoneVerify");
                        if (phoneVerify!=null){
                            tvPhoneVerify.setText(phoneVerify);
                        }else {
                            tvPhoneVerify.setText("");
                        }

                        //手机验证提示内容
                        TextView tvPhoneVerifyTips = (TextView) findViewById(R.id.tv_one_key_verify_phone_verify_tips);
                        String phoneVerifyTips=verifyMessage.get("phoneVerifyTips");
                        if (phoneVerifyTips!=null){
                            tvPhoneVerifyTips.setText(verifyMessage.get("phoneVerifyTips"));
                        }else {
                            tvPhoneVerifyTips.setText("");
                        }

                        //其他手机验证
                        TextView tvOtherPhoneVerify = (TextView) findViewById(R.id.tv_one_key_verify_other_phone);
                        String otherVerify=verifyMessage.get("otherVerify");
                        if (otherVerify!=null){
                            tvOtherPhoneVerify.setText(otherVerify);
                        }else {
                            tvOtherPhoneVerify.setText("");
                        }


                    }
                })
                .build());
        phoneNumberAuthHelper.setUIClickListener(new AuthUIControlClickListener() {
            @Override
            public void onClick(String code, Context context, JSONObject jsonObject) {
                String json = jsonObject == null ? "" : jsonObject.toJSONString();
                Log.i(TAG, "code:" + code + "  json:" + json);
            }
        });

        int authPageOrientation = ActivityInfo.SCREEN_ORIENTATION_SENSOR_PORTRAIT;
        if (Build.VERSION.SDK_INT == 26) {
            authPageOrientation = ActivityInfo.SCREEN_ORIENTATION_BEHIND;
        }

        //一键验证文本
        String verifyBtn=verifyMessage.get("verifyBtn");
        if (verifyBtn==null){
            verifyBtn="本机号码一键验证";
        }

        AuthUIConfig.Builder builder = new AuthUIConfig.Builder()

                .setAuthPageActIn("one_key_verify_in_activity", "one_key_verify_in_activity")
                .setAuthPageActOut("one_key_verify_in_activity", "one_key_verify_in_activity")
                //一键验证按钮设置
                .setLogBtnHeight(52)
                .setLogBtnOffsetY(325)
                .setLogBtnMarginLeftAndRight(28)
                .setLogBtnText(verifyBtn)
                .setLogBtnTextSize(18)
                .setLogBtnBackgroundPath("drawable/one_key_login_click")
                //手机号码设置
                .setNumFieldOffsetY(172)
                .setNumberSize(36)
                .setNumberColor(0xff333333)

                .setSloganOffsetY(230)
                .setSloganTextColor(0x661e222b)
                //设置屏幕方向
                .setScreenOrientation(authPageOrientation)
                //设置隐藏
                .setCheckboxHidden(true)
                .setSwitchAccHidden(true)
                .setNavHidden(true)
                .setLogoHidden(true);


        //协议
        String onePrivacy=verifyMessage.get("onePrivacy");
        String onePrivacyUrl=verifyMessage.get("onePrivacyUrl");
        String twoPrivacy=verifyMessage.get("twoPrivacy");
        String twoPrivacyUrl=verifyMessage.get("twoPrivacyUrl");
        String prePrivacy=verifyMessage.get("prePrivacy");

        if (!TextUtils.isEmpty(prePrivacy)){
            builder.setPrivacyBefore(prePrivacy);
        }
        if (!TextUtils.isEmpty(onePrivacy)){
            builder.setAppPrivacyOne(onePrivacy,onePrivacyUrl!=null?onePrivacyUrl:"");
        }
        if (!TextUtils.isEmpty(twoPrivacy)){
            builder.setAppPrivacyOne(twoPrivacy,twoPrivacyUrl!=null?twoPrivacyUrl:"");
        }

        phoneNumberAuthHelper.setAuthUIConfig(builder.create());

    }

    @Override
    public void onLoginLayout(PhoneNumberAuthHelper phoneNumberAuthHelper) {
        phoneNumberAuthHelper.removeAuthRegisterXmlConfig();
        phoneNumberAuthHelper.removeAuthRegisterViewConfig();
        phoneNumberAuthHelper.addAuthRegisterXmlConfig(new AuthRegisterXmlConfig.Builder()
                .setLayout(R.layout.activity_one_key_login, new AbstractPnsViewDelegate() {
                    @Override
                    public void onViewCreated(View view) {
                        //返回
                        RelativeLayout rlBack = (RelativeLayout) findViewById(R.id.rv_one_key_login_auth_back);
                        rlBack.setOnClickListener(FxchatOneKeyHelper.this);
                        //其他方式登录
                        TextView tvOtherLogin = (TextView) findViewById(R.id.tv_one_key_login_auth_login);
                        tvOtherLogin.setOnClickListener(FxchatOneKeyHelper.this);
                        //设置文本
                        String otherLogin=loginMessage.get("otherLogin");
                        if (otherLogin!=null){
                            tvOtherLogin.setText(otherLogin);
                        }else {
                            tvOtherLogin.setText("");
                        }
                        //点击微信
                        ImageView weChat = (ImageView) findViewById(R.id.iv_one_key_login_auth_wechat);
                        weChat.setOnClickListener(FxchatOneKeyHelper.this);
                        //点击QQ
                        ImageView qq = (ImageView) findViewById(R.id.iv_one_key_login_auth_qq);
                        qq.setOnClickListener(FxchatOneKeyHelper.this);
                        //点击微博
                        ImageView weibo = (ImageView) findViewById(R.id.iv_one_key_login_auth_weibo);
                        weibo.setOnClickListener(FxchatOneKeyHelper.this);

                    }
                })
                .build());
        phoneNumberAuthHelper.setUIClickListener(new AuthUIControlClickListener() {
            @Override
            public void onClick(String code, Context context, JSONObject jsonObject) {
                String json = jsonObject == null ? "" : jsonObject.toJSONString();
                Log.i(TAG, "code:" + code + "  json:" + json);
            }
        });

        int authPageOrientation = ActivityInfo.SCREEN_ORIENTATION_SENSOR_PORTRAIT;
        if (Build.VERSION.SDK_INT == 26) {
            authPageOrientation = ActivityInfo.SCREEN_ORIENTATION_BEHIND;
        }


        //一键验证文本
        String loginBtn=loginMessage.get("loginBtn");
        if (loginBtn==null){
            loginBtn="本机一键登录";
        }

        AuthUIConfig.Builder builder = new AuthUIConfig.Builder()

                .setAuthPageActIn("one_key_login_in_activity", "one_key_login_out_activity")
                .setAuthPageActOut("one_key_login_in_activity", "one_key_login_out_activity")

                .setLogBtnHeight(52)
                .setLogBtnOffsetY(255)
                .setLogBtnMarginLeftAndRight(28)
                .setLogBtnText(loginBtn)
                .setLogBtnTextSize(18)
                .setLogBtnBackgroundPath("drawable/one_key_login_click")

                .setNumFieldOffsetY(127)
                .setNumberSize(36)
                .setNumberColor(0xff333333)

                .setSloganOffsetY(183)
                .setSloganTextColor(0x661e222b)

                .setScreenOrientation(authPageOrientation)
                .setPrivacyOffsetY_B(28)
                .setPrivacyMargin(65)
                .setPrivacyTextSize(12)
                .setAppPrivacyColor(0x661e222b, 0x991e222b)

                .setCheckboxHidden(true)
                .setSwitchAccHidden(true)
                .setNavHidden(true)
                .setLogoHidden(true);

        //协议
        String prePrivacy=loginMessage.get("prePrivacy");
        String onePrivacy=loginMessage.get("onePrivacy");
        String onePrivacyUrl=loginMessage.get("onePrivacyUrl");
        String twoPrivacy=loginMessage.get("twoPrivacy");
        String twoPrivacyUrl=loginMessage.get("twoPrivacyUrl");


        if (!TextUtils.isEmpty(prePrivacy)){
            builder.setPrivacyBefore(prePrivacy);
        }
        if (!TextUtils.isEmpty(onePrivacy)){
            builder.setAppPrivacyOne(onePrivacy,onePrivacyUrl!=null?onePrivacyUrl:"");
        }
        if (!TextUtils.isEmpty(twoPrivacy)){
            builder.setAppPrivacyOne(twoPrivacy,twoPrivacyUrl!=null?twoPrivacyUrl:"");
        }

        phoneNumberAuthHelper.setAuthUIConfig(builder.create());
    }

    @Override
    public void onClick(View v) {

        ViewMessageResult result = new ViewMessageResult();
        result.setPosition(-1);
        //一键登录
        if (v.getId() == R.id.iv_one_key_login_auth_wechat) {
            result.setPosition(weChat);
        } else if (v.getId() == R.id.iv_one_key_login_auth_qq) {
            result.setPosition(qq);
        } else if (v.getId() == R.id.iv_one_key_login_auth_weibo) {
            result.setPosition(weibo);
        }else if (v.getId()==R.id.tv_one_key_login_auth_login){
            result.setPosition(other_login);
            quitLoginPage();
        }else if (v.getId()==R.id.rv_one_key_login_auth_back){
            result.setPosition(back);
            quitLoginPage();
        }

        //一键验证
        if (v.getId()==R.id.tv_one_key_verify_other_phone){
            result.setPosition(other_phone_verify);
            quitLoginPage();
        }

        viewEvent.sendSuccess(result);
    }
}
