import 'dart:io';
import 'package:flutter_ali_oauth/config/AliOauthConfig.dart';
import 'package:flutter_ali_oauth/flutter_ali_oauth.dart';

class OneKeyLoginHelper {
  static final String TAG = "OneKeyLoginHelper";

  /// 一键登录秘钥
  static String _iosAliAuthSdk =
      '';
  static String _androidAuthSdk =
      '';

  ///初始化
  static init({String androidSk, String iosSk}) {
    _androidAuthSdk = androidSk ?? _androidAuthSdk;
    _iosAliAuthSdk = iosSk ?? _iosAliAuthSdk;
    // 初始化插件
    if (Platform.isAndroid) {
      FlutterAliOauth.initSdk(_androidAuthSdk);
    } else {
      FlutterAliOauth.initSdk(_iosAliAuthSdk);
    }
  }

  ///一键登录
  static login(OneKeyLoginMessage loginMessage) async {
    bool checkVerifyEnable = false;
    try {
      checkVerifyEnable = await FlutterAliOauth.checkVerifyEnable;

      if (checkVerifyEnable) {
        FlutterAliOauth.login(
            type: AliOauthType.LOGIN, oneKeyLoginMessage: loginMessage);
      }
    } catch (e) {
      print("$TAG ：_quickLogin error:" + e);
    }
  }

  ///一键验证
  static verify(OneKeyVerifyMessage oneKeyVerifyMessage) async {
    bool checkVerifyEnable = false;
    try {
      checkVerifyEnable = await FlutterAliOauth.checkVerifyEnable;

      if (checkVerifyEnable) {
        FlutterAliOauth.login(
            type: AliOauthType.VERIFY,
            oneKeyVerifyMessage: oneKeyVerifyMessage);
      }
    } catch (e) {
      print("$TAG ：_quickVerify error:" + e);
    }
  }

  static Future<bool> checkVerifyEnable() async {
    return await FlutterAliOauth.checkVerifyEnable;
  }

  /// 预取号
  static Future<dynamic> preLogin({int timeOut = 10000}) async {
    return await FlutterAliOauth.preLogin(timeOut: timeOut);
  }

  ///隐藏Loading
  static hideLoading() async {
    FlutterAliOauth.hideLoginLoading;
  }

  ///离开登录/验证页面
  static quitLoginPage() async {
    FlutterAliOauth.quitLoginPage;
  }

  /// 获取一键验证token
  static Future<dynamic> getVerifyToken({int timeOut = 10000}) async {
    return await FlutterAliOauth.getVerifyToken(timeOut: timeOut);
  }

  ///View点击事件监听
  static setViewListener(dynamic _onViewEvent) {
    FlutterAliOauth.getViewChannel
        .receiveBroadcastStream()
        .listen(_onViewEvent);
  }

  ///Token回调事件监听
  static setTokenListener(dynamic _onTokenSuccess) {
    FlutterAliOauth.getTokenChannel
        .receiveBroadcastStream()
        //onError 平台出现问题，暂时无法解决
        .listen(_onTokenSuccess);
  }
}
