
import 'dart:async';
import 'package:flutter/services.dart';
import 'package:flutter_ali_oauth/model/one_key_message.dart';
import 'config/AliOauthConfig.dart';
export 'model/view_message.dart';
export 'click_event.dart';
export 'ali_oauth_utils.dart';
export 'model/one_key_message.dart';
export 'model/accelerate_token_result.dart';
export 'model/token_result.dart';

class FlutterAliOauth {

  static final String CODE_GET_TOKEN_SUCCESS= "600000";
  /**
   * 打开授权页面成功
   */
  static final String CODE_CALL_AUTHORIZE = "600001";
  /**
   * 唤起授权页失败
   */
  static final String CODE_CALL_AUTHORIZE_FAIL = "600002";
  /**
   * 未检测到sim卡
   */
  static final String CODE_NOT_SIM = "600007";
  /**
   * 蜂窝网络未开启
   */
  static final String CODE_NOT_PHONE_NET = "600008";
  /**
   * 未知异常
   */
  static final String CODE_UNKONW_EXP = "600010";
  /**
   * 获取token失败
   */
  static final String CODE_GET_TOKEN_FAIL = "600011";
  /**
   * 预取号失败
   */
  static final String CODE_PRE_GET_PHONE_FAIL = "600012";
  /**
   * 接口超时
   */
  static final String CODE_API_TIMEOUT = "600015";
  /**
   * 用户已切换上网卡
   */
  static final String CODE_CHANG_SIM = "600021";

  /**
   * 用户取消登录
   */
  static final String CODE_CANCEL_LOGIN = "700000";

  /**
   * 网络无连接
   */
  static final String CODE_NOT_NET = "-8001";

  /**
   * 当前网络不支持，请检测蜂窝网络后重试
   */
  static final String CODE_NOT_SUPPOR = "10000";

  //channel
  static const String _MAIN_CHANNEL_NAME="com.stl/stl_ali_oauth";

  static const String VIEW_POSITION_CHANNEL_NAME="com.stl/viewEvent";

  static const String TOKEN_CHANNEL_NAME="com.stl/tokenEvent";

  static const MethodChannel _channel = const MethodChannel(_MAIN_CHANNEL_NAME);

  static const mViewEventChannel = const EventChannel(VIEW_POSITION_CHANNEL_NAME);

  static const mTokenEventChannel = const EventChannel(TOKEN_CHANNEL_NAME);

  /// 初始化SDK
  static Future<dynamic> initSdk(String sk) async {
    Map<String, String> params = {AliOauthConfig.SK: sk};
    return await _channel.invokeMethod("init", params);
  }

  /// SDK判断网络环境是否支持
  static Future<bool> get checkVerifyEnable async {
    return await _channel.invokeMethod("checkVerifyEnable");
  }

  /// 唤起一键登录/一键验证界面
  static Future<dynamic> login({int timeOut=10000,int type=AliOauthType.LOGIN,OneKeyLoginMessage oneKeyLoginMessage,OneKeyVerifyMessage oneKeyVerifyMessage}) async {
    Map<String, Object> params;
    if(type==AliOauthType.LOGIN){
     params = {AliOauthConfig.TIME_OUT: timeOut,AliOauthConfig.TYPE:type,AliOauthConfig.ONE_KEY_LOGIN_MESSAGE:oneKeyLoginMessage.toJson()};
    }else{
      params = {AliOauthConfig.TIME_OUT: timeOut,AliOauthConfig.TYPE:type,AliOauthConfig.ONE_KEY_VETIFY_MESSAGE:oneKeyVerifyMessage.toJson()};
    }
    return await _channel.invokeMethod('login',params);
  }
  /// 获取一键验证token
  static Future<dynamic> getVerifyToken({int timeOut=10000}) async {
    Map<String, int> params = {AliOauthConfig.TIME_OUT: timeOut,};

    return await _channel.invokeMethod('getVerifyToken',params);
  }

  /// 预取号
  static Future<dynamic>  preLogin({int timeOut=10000}) async {
    Map<String, int> params = {AliOauthConfig.TIME_OUT: timeOut};
    return await _channel.invokeMethod('preLogin',params);
  }

  /// 离开登录页面
  static Future<dynamic>  get quitLoginPage async {
    return await _channel.invokeMethod('quitLoginPage');
  }

  /// 隐藏Loading
  static Future<dynamic>  get hideLoginLoading async {
    return await _channel.invokeMethod('hideLoginLoading');
  }


  /// 获取View的事件推送
  static EventChannel  get getViewChannel  {
    return  mViewEventChannel;
  }

  /// 获取Token的事件推送
  static EventChannel  get getTokenChannel  {
    return  mTokenEventChannel;
  }

}
